@@ -19,11 +19,64 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-lame
+ * @short_description: an encoder that encodes audio to MPEG-1 layer 3 (mp3)
+ * @see_also: mad, vorbisenc
+ *
+ * <refsect2>
+ * <para>
+ * This element encodes raw integer audio into an MPEG-1 layer 3 (MP3) stream.
+ * Note that <ulink url="http://en.wikipedia.org/wiki/MP3">MP3</ulink> is not
+ * a free format, there are licensing and patent issues to take into
+ * consideration. See <ulink url="http://www.vorbis.com/">Ogg/Vorbis</ulink>
+ * for a royalty free (and often higher quality) alternative.
+ * </para>
+ * <title>Writing metadata (tags)</title>
+ * <para>
+ * Whilst the lame encoder element does claim to implement the GstTagSetter
+ * interface, it does so only for backwards compatibility reasons. Tag writing
+ * has been removed from lame. Use external elements like id3v2mux or apev2mux
+ * to add tags to your MP3 streams. The same goes for XING headers: use the
+ * xingmux element to add XING headers to your VBR mp3 file.
+ * </para>
+ * <title>Example pipelines</title>
+ * <para>
+ * Encode a test sine signal to MP3.
+ * </para>
+ * <programlisting>
+ * gst-launch -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! lame ! filesink location=sine.mp3
+ * </programlisting>
+ * <para>
+ * Record from a sound card using ALSA and encode to MP3
+ * </para>
+ * <programlisting>
+ * gst-launch -v alsasrc ! audioconvert ! lame bitrate=192 ! filesink location=alsasrc.mp3
+ * </programlisting>
+ * <para>
+ * Transcode from a .wav file to MP3 (the id3v2mux element is optional):
+ * </para>
+ * <programlisting>
+ * gst-launch -v filesrc location=music.wav ! decodebin ! audioconvert ! audioresample ! lame bitrate=192 ! id3v2mux ! filesink location=music.mp3
+ * </programlisting>
+ * <para>
+ * Encode Audio CD track 5 to MP3:
+ * </para>
+ * <programlisting>
+ * gst-launch -v cdda://5 ! audioconvert ! lame bitrate=192 ! filesink location=track5.mp3
+ * </programlisting>
+ * </refsect2>
+ *
+ * Last reviewed on 2006-10-13 (0.10.4)
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include "string.h"
 #include "gstlame.h"
+#include "gst/gst-i18n-plugin.h"
 
 #ifdef lame_set_preset
 #define GST_LAME_PRESET
@@ -32,6 +85,10 @@
 GST_DEBUG_CATEGORY_STATIC (debug);
 #define GST_CAT_DEFAULT debug
 
+#define DEFAULT_MIN_VBR_BITRATE 112
+#define DEFAULT_MAX_VBR_BITRATE 160
+#define DEFAULT_MEAN_VBR_BITRATE 128
+
 /* elementfactory information */
 static GstElementDetails gst_lame_details = {
   "L.A.M.E. mp3 encoder",
@@ -313,7 +370,9 @@ gst_lame_class_init (GstLameClass * klass)
   gobject_class->finalize = gst_lame_finalize;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BITRATE,
-      g_param_spec_int ("bitrate", "Bitrate (kb/s)", "Bitrate in kbit/sec",
+      g_param_spec_int ("bitrate", "Bitrate (kb/s)",
+          "Bitrate in kbit/sec (8, 16, 24, 32, 40, 48, 56, 64, 80, 96, "
+          "112, 128, 160, 192, 224, 256 or 320)",
           8, 320, 128, G_PARAM_READWRITE));
   /* compression ratio set to 0.0 by default otherwise it overrides the bitrate setting */
   g_object_class_install_property (G_OBJECT_CLASS (klass),
@@ -329,10 +388,10 @@ gst_lame_class_init (GstLameClass * klass)
       g_param_spec_enum ("mode", "Mode", "Encoding mode", GST_TYPE_LAME_MODE, 0,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FORCE_MS,
-      g_param_spec_boolean ("force_ms", "Force ms",
+      g_param_spec_boolean ("force-ms", "Force ms",
           "Force ms_stereo on all frames", TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FREE_FORMAT,
-      g_param_spec_boolean ("free_format", "Free format",
+      g_param_spec_boolean ("free-format", "Free format",
           "Produce a free format bitstream", TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_COPYRIGHT,
       g_param_spec_boolean ("copyright", "Copyright", "Mark as copyright", TRUE,
@@ -341,78 +400,84 @@ gst_lame_class_init (GstLameClass * klass)
       g_param_spec_boolean ("original", "Original", "Mark as non-original",
           TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ERROR_PROTECTION,
-      g_param_spec_boolean ("error_protection", "Error protection",
+      g_param_spec_boolean ("error-protection", "Error protection",
           "Adds 16 bit checksum to every frame", TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PADDING_TYPE,
-      g_param_spec_enum ("padding_type", "Padding type", "Padding type",
+      g_param_spec_enum ("padding-type", "Padding type", "Padding type",
           GST_TYPE_LAME_PADDING, 0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_EXTENSION,
       g_param_spec_boolean ("extension", "Extension", "Extension", TRUE,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_STRICT_ISO,
-      g_param_spec_boolean ("strict_iso", "Strict ISO",
+      g_param_spec_boolean ("strict-iso", "Strict ISO",
           "Comply as much as possible to ISO MPEG spec", TRUE,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass),
-      ARG_DISABLE_RESERVOIR, g_param_spec_boolean ("disable_reservoir",
+      ARG_DISABLE_RESERVOIR, g_param_spec_boolean ("disable-reservoir",
           "Disable reservoir", "Disable the bit reservoir", TRUE,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR,
       g_param_spec_enum ("vbr", "VBR", "Specify bitrate mode",
           GST_TYPE_LAME_VBRMODE, vbr_off, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR_QUALITY,
-      g_param_spec_enum ("vbr_quality", "VBR Quality", "VBR Quality",
+      g_param_spec_enum ("vbr-quality", "VBR Quality", "VBR Quality",
           GST_TYPE_LAME_QUALITY, 5, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR_MEAN_BITRATE,
-      g_param_spec_int ("vbr_mean_bitrate", "VBR mean bitrate",
-          "Specify mean bitrate", 0, G_MAXINT, 0, G_PARAM_READWRITE));
+      g_param_spec_int ("vbr-mean-bitrate", "VBR mean bitrate",
+          "Specify mean VBR bitrate", 8, 320,
+          DEFAULT_MEAN_VBR_BITRATE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR_MIN_BITRATE,
-      g_param_spec_int ("vbr_min_bitrate", "VBR min bitrate",
-          "Specify min bitrate", 0, G_MAXINT, 0, G_PARAM_READWRITE));
+      g_param_spec_int ("vbr-min-bitrate", "VBR min bitrate",
+          "Specify minimum VBR bitrate (8, 16, 24, 32, 40, 48, 56, 64, 80, 96, "
+          "112, 128, 160, 192, 224, 256 or 320)",
+          8, 320, DEFAULT_MIN_VBR_BITRATE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR_MAX_BITRATE,
-      g_param_spec_int ("vbr_max_bitrate", "VBR max bitrate",
-          "Specify max bitrate", 0, G_MAXINT, 0, G_PARAM_READWRITE));
+      g_param_spec_int ("vbr-max-bitrate", "VBR max bitrate",
+          "Specify maximum VBR bitrate (8, 16, 24, 32, 40, 48, 56, 64, 80, 96, "
+          "112, 128, 160, 192, 224, 256 or 320)",
+          8, 320, DEFAULT_MAX_VBR_BITRATE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR_HARD_MIN,
-      g_param_spec_int ("vbr_hard_min", "VBR hard min",
-          "Specify hard min bitrate", 0, G_MAXINT, 0, G_PARAM_READWRITE));
+      g_param_spec_int ("vbr-hard-min", "VBR hard min",
+          "Specify whether min VBR bitrate is a hard limit. Normally, "
+          "it can be violated for silence", 0, 1, 0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LOWPASS_FREQ,
-      g_param_spec_int ("lowpass_freq", "Lowpass freq",
+      g_param_spec_int ("lowpass-freq", "Lowpass freq",
           "frequency(kHz), lowpass filter cutoff above freq", 0, 50000, 0,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LOWPASS_WIDTH,
-      g_param_spec_int ("lowpass_width", "Lowpass width",
+      g_param_spec_int ("lowpass-width", "Lowpass width",
           "frequency(kHz) - default 15% of lowpass freq", 0, G_MAXINT, 0,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HIGHPASS_FREQ,
-      g_param_spec_int ("highpass_freq", "Highpass freq",
+      g_param_spec_int ("highpass-freq", "Highpass freq",
           "frequency(kHz), highpass filter cutoff below freq", 0, 50000, 0,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HIGHPASS_WIDTH,
-      g_param_spec_int ("highpass_width", "Highpass width",
+      g_param_spec_int ("highpass-width", "Highpass width",
           "frequency(kHz) - default 15% of highpass freq", 0, G_MAXINT, 0,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ATH_ONLY,
-      g_param_spec_boolean ("ath_only", "ATH only",
+      g_param_spec_boolean ("ath-only", "ATH only",
           "Ignore GPSYCHO completely, use ATH only", TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ATH_SHORT,
-      g_param_spec_boolean ("ath_short", "ATH short",
+      g_param_spec_boolean ("ath-short", "ATH short",
           "Ignore GPSYCHO for short blocks, use ATH only", TRUE,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NO_ATH,
-      g_param_spec_boolean ("no_ath", "No ath",
+      g_param_spec_boolean ("no-ath", "No ath",
           "turns ATH down to a flat noise floor", TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ATH_LOWER,
-      g_param_spec_int ("ath_lower", "ATH lower", "lowers ATH by x dB",
+      g_param_spec_int ("ath-lower", "ATH lower", "lowers ATH by x dB",
           G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_CWLIMIT,
       g_param_spec_int ("cwlimit", "Cwlimit",
           "Compute tonality up to freq (in kHz) default 8.8717", 0, 50000, 0,
           G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_ALLOW_DIFF_SHORT,
-      g_param_spec_boolean ("allow_diff_short", "Allow diff short",
+      g_param_spec_boolean ("allow-diff-short", "Allow diff short",
           "Allow diff short", TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NO_SHORT_BLOCKS,
-      g_param_spec_boolean ("no_short_blocks", "No short blocks",
+      g_param_spec_boolean ("no-short-blocks", "No short blocks",
           "Do not use short blocks", TRUE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_EMPHASIS,
       g_param_spec_boolean ("emphasis", "Emphasis", "Emphasis", TRUE,
@@ -438,7 +503,7 @@ gst_lame_sink_setcaps (GstPad * pad, GstCaps * caps)
   GstStructure *structure;
   GstCaps *othercaps;
 
-  lame = GST_LAME (gst_pad_get_parent (pad));
+  lame = GST_LAME (GST_PAD_PARENT (pad));
   structure = gst_caps_get_structure (caps, 0);
 
   if (!gst_structure_get_int (structure, "rate", &lame->samplerate))
@@ -446,7 +511,7 @@ gst_lame_sink_setcaps (GstPad * pad, GstCaps * caps)
   if (!gst_structure_get_int (structure, "channels", &lame->num_channels))
     goto no_channels;
 
-  GST_DEBUG_OBJECT (lame, "sink_setcaps, setting up lame");
+  GST_DEBUG_OBJECT (lame, "setting up lame");
   if (!gst_lame_setup (lame))
     goto setup_failed;
 
@@ -454,6 +519,11 @@ gst_lame_sink_setcaps (GstPad * pad, GstCaps * caps)
   out_samplerate = lame_get_out_samplerate (lame->lgf);
   if (out_samplerate == 0)
     goto zero_output_rate;
+  if (out_samplerate != lame->samplerate) {
+    GST_WARNING_OBJECT (lame,
+        "output samplerate %d is different from incoming samplerate %d",
+        out_samplerate, lame->samplerate);
+  }
 
   othercaps =
       gst_caps_new_simple ("audio/mpeg",
@@ -465,36 +535,29 @@ gst_lame_sink_setcaps (GstPad * pad, GstCaps * caps)
   /* and use these caps */
   gst_pad_set_caps (lame->srcpad, othercaps);
   gst_caps_unref (othercaps);
-  gst_object_unref (lame);
 
   return TRUE;
 
 no_rate:
   {
-    GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
-        ("no rate specified in input"));
-    gst_object_unref (lame);
+    GST_ERROR_OBJECT (lame, "input caps have no sample rate field");
     return FALSE;
   }
 no_channels:
   {
-    GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
-        ("no channels specified in input"));
-    gst_object_unref (lame);
+    GST_ERROR_OBJECT (lame, "input caps have no channels field");
     return FALSE;
   }
 zero_output_rate:
   {
-    GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
-        ("lame decided on a zero sample rate"));
-    gst_object_unref (lame);
+    GST_ELEMENT_ERROR (lame, LIBRARY, SETTINGS, (NULL),
+        ("LAME decided on a zero sample rate"));
     return FALSE;
   }
 setup_failed:
   {
-    GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
-        ("could not initialize encoder (wrong parameters?)"));
-    gst_object_unref (lame);
+    GST_ELEMENT_ERROR (lame, LIBRARY, SETTINGS,
+        (_("Failed to configure LAME encoder. Check your encoding parameters.")), NULL);
     return FALSE;
   }
 }
@@ -520,6 +583,11 @@ gst_lame_init (GstLame * lame)
 
   /* create an encoder state so we can ask about defaults */
   lame->lgf = lame_init ();
+  if (lame->lgf == NULL)
+    goto init_error;
+
+  if (lame_init_params (lame->lgf) < 0)
+    goto init_error;
 
   lame->samplerate = 44100;
   lame->num_channels = 2;
@@ -544,10 +612,18 @@ gst_lame_init (GstLame * lame)
   lame->disable_reservoir = lame_get_disable_reservoir (lame->lgf);
   lame->vbr = vbr_off;          /* lame_get_VBR (lame->lgf); */
   lame->vbr_quality = 5;
+#if 0
+  /* Replaced by our own more informative constants, 
+     rather than LAME's defaults */
   lame->vbr_mean_bitrate = lame_get_VBR_mean_bitrate_kbps (lame->lgf);
   lame->vbr_min_bitrate = lame_get_VBR_min_bitrate_kbps (lame->lgf);
   lame->vbr_max_bitrate = 0;    /* lame_get_VBR_max_bitrate_kbps (lame->lgf);
                                  * => 0/no vbr possible */
+#else
+  lame->vbr_mean_bitrate = DEFAULT_MEAN_VBR_BITRATE;
+  lame->vbr_min_bitrate = DEFAULT_MIN_VBR_BITRATE;
+  lame->vbr_max_bitrate = DEFAULT_MAX_VBR_BITRATE;
+#endif
   lame->vbr_hard_min = lame_get_VBR_hard_min (lame->lgf);
   /* lame->lowpass_freq = 50000;    lame_get_lowpassfreq (lame->lgf);
    * => 0/lowpass on everything ? */
@@ -571,16 +647,64 @@ gst_lame_init (GstLame * lame)
   lame->lgf = NULL;
 
   GST_DEBUG_OBJECT (lame, "done initializing");
+  lame->init_error = FALSE;
+  return;
+
+/* ERRORS */
+init_error:
+  {
+    GST_ERROR_OBJECT (lame, "error initializing");
+    lame->init_error = TRUE;
+    if (lame->lgf) {
+      lame_close (lame->lgf);
+      lame->lgf = NULL;
+    }
+  }
 }
 
+/* <php-emulation-mode>three underscores for ___rate is really really really
+ * private as opposed to one underscore<php-emulation-mode> */
+/* call this MACRO outside of the NULL state so that we have a higher chance
+ * of actually having a pipeline and bus to get the message through */
+
+#define CHECK_AND_FIXUP_BITRATE(obj,param,rate,free_format) 		  \
+G_STMT_START {                                                            \
+  gint ___rate = rate;                                                    \
+  gint maxrate = 320;							  \
+  gint multiplier = 64;							  \
+  if (!free_format) {                                                     \
+    if (rate <= 64) {							  \
+      maxrate = 64; multiplier = 8;                                       \
+      if ((rate % 8) != 0) ___rate = GST_ROUND_UP_8 (rate); 		  \
+    } else if (rate <= 128) {						  \
+      maxrate = 128; multiplier = 16;                                     \
+      if ((rate % 16) != 0) ___rate = GST_ROUND_UP_16 (rate);             \
+    } else if (rate <= 256) {						  \
+      maxrate = 256; multiplier = 32;                                     \
+      if ((rate % 32) != 0) ___rate = GST_ROUND_UP_32 (rate);             \
+    } else if (rate <= 320) { 						  \
+      maxrate = 320; multiplier = 64;                                     \
+      if ((rate % 64) != 0) ___rate = GST_ROUND_UP_64 (rate);             \
+    }                                                                     \
+    if (___rate != rate) {                                                \
+      GST_ELEMENT_WARNING (obj, LIBRARY, SETTINGS,			  \
+          (_("The requested bitrate %d kbit/s for property '%s' "         \
+             "is not allowed. "  					  \
+            "The bitrate was changed to %d kbit/s."), rate,		  \
+	    param,  ___rate), 					          \
+          ("A bitrate below %d should be a multiple of %d.", 		  \
+              maxrate, multiplier));		  			  \
+      rate = ___rate;                                                     \
+    }                                                                     \
+  }                                                                       \
+} G_STMT_END
+
 static void
 gst_lame_set_property (GObject * object, guint prop_id, const GValue * value,
     GParamSpec * pspec)
 {
   GstLame *lame;
 
-  g_return_if_fail (GST_IS_LAME (object));
-
   lame = GST_LAME (object);
 
   switch (prop_id) {
@@ -696,8 +820,6 @@ gst_lame_get_property (GObject * object, guint prop_id, GValue * value,
 {
   GstLame *lame;
 
-  g_return_if_fail (GST_IS_LAME (object));
-
   lame = GST_LAME (object);
 
   switch (prop_id) {
@@ -826,13 +948,29 @@ gst_lame_sink_event (GstPad * pad, GstEvent * event)
         buf = gst_buffer_new_and_alloc (7200);
         size = lame_encode_flush (lame->lgf, GST_BUFFER_DATA (buf), 7200);
 
-        if (size > 0) {
+        if (size > 0 && lame->last_flow == GST_FLOW_OK) {
+          gint64 duration;
+
+          duration = gst_util_uint64_scale_int (size, GST_SECOND,
+              2 * lame->samplerate * lame->num_channels);
+
+          if (lame->last_ts == GST_CLOCK_TIME_NONE) {
+            lame->last_ts = lame->eos_ts;
+            lame->last_duration = duration;
+          } else {
+            lame->last_duration += duration;
+          }
+
+          GST_BUFFER_TIMESTAMP (buf) = lame->last_ts;
+          GST_BUFFER_DURATION (buf) = lame->last_duration;
+          lame->last_ts = GST_CLOCK_TIME_NONE;
           GST_BUFFER_SIZE (buf) = size;
           GST_DEBUG_OBJECT (lame, "pushing final packet of %u bytes", size);
           gst_buffer_set_caps (buf, GST_PAD_CAPS (lame->srcpad));
           gst_pad_push (lame->srcpad, buf);
         } else {
-          GST_DEBUG_OBJECT (lame, "no final packet (size=%d)", size);
+          GST_DEBUG_OBJECT (lame, "no final packet (size=%d, last_flow=%s)",
+              size, gst_flow_get_name (lame->last_flow));
           gst_buffer_unref (buf);
         }
       }
@@ -884,10 +1022,13 @@ gst_lame_chain (GstPad * pad, GstBuffer * buf)
   guint8 *data;
   guint size;
 
-  lame = GST_LAME (gst_pad_get_parent (pad));
+  lame = GST_LAME (GST_PAD_PARENT (pad));
 
   GST_LOG_OBJECT (lame, "entered chain");
 
+  if (lame->init_error)
+    goto init_error;
+
   if (!lame->setup)
     goto not_setup;
 
@@ -912,17 +1053,18 @@ gst_lame_chain (GstPad * pad, GstBuffer * buf)
   }
 
   GST_LOG_OBJECT (lame, "encoded %d bytes of audio to %d bytes of mp3",
-      GST_BUFFER_SIZE (buf), mp3_size);
+      size, mp3_size);
 
   duration = gst_util_uint64_scale_int (size, GST_SECOND,
       2 * lame->samplerate * lame->num_channels);
 
   if (GST_BUFFER_DURATION (buf) != GST_CLOCK_TIME_NONE &&
-      GST_BUFFER_DURATION (buf) != duration)
+      GST_BUFFER_DURATION (buf) != duration) {
     GST_DEBUG_OBJECT (lame, "incoming buffer had incorrect duration %"
-        GST_TIME_FORMAT "outgoing buffer will have correct duration %"
+        GST_TIME_FORMAT ", outgoing buffer will have correct duration %"
         GST_TIME_FORMAT,
         GST_TIME_ARGS (GST_BUFFER_DURATION (buf)), GST_TIME_ARGS (duration));
+  }
 
   if (lame->last_ts == GST_CLOCK_TIME_NONE) {
     lame->last_ts = GST_BUFFER_TIMESTAMP (buf);
@@ -951,13 +1093,20 @@ gst_lame_chain (GstPad * pad, GstBuffer * buf)
     gst_buffer_set_caps (outbuf, GST_PAD_CAPS (lame->srcpad));
 
     result = gst_pad_push (lame->srcpad, outbuf);
+    lame->last_flow = result;
+    if (result != GST_FLOW_OK) {
+      GST_DEBUG_OBJECT (lame, "flow return: %s", gst_flow_get_name (result));
+    }
 
+    if (GST_CLOCK_TIME_IS_VALID (lame->last_ts))
+      lame->eos_ts = lame->last_ts + lame->last_duration;
+    else
+      lame->eos_ts = GST_CLOCK_TIME_NONE;
     lame->last_ts = GST_CLOCK_TIME_NONE;
   } else {
     g_free (mp3_data);
     result = GST_FLOW_OK;
   }
-  gst_object_unref (lame);
 
   return result;
 
@@ -967,7 +1116,12 @@ not_setup:
     gst_buffer_unref (buf);
     GST_ELEMENT_ERROR (lame, CORE, NEGOTIATION, (NULL),
         ("encoder not initialized (input is not audio?)"));
-    gst_object_unref (lame);
+    return GST_FLOW_ERROR;
+  }
+init_error:
+  {
+    gst_buffer_unref (buf);
+    GST_ELEMENT_ERROR (lame, LIBRARY, INIT, (NULL), (NULL));
     return GST_FLOW_ERROR;
   }
 }
@@ -996,6 +1150,9 @@ gst_lame_setup (GstLame * lame)
 
   lame->lgf = lame_init ();
 
+  if (lame->lgf == NULL)
+    return FALSE;
+
   /* let lame choose a default samplerate */
   lame_set_out_samplerate (lame->lgf, 0);
 
@@ -1007,6 +1164,7 @@ gst_lame_setup (GstLame * lame)
     lame->mode = 3;
 
   CHECK_ERROR (lame_set_num_channels (lame->lgf, lame->num_channels));
+  CHECK_AND_FIXUP_BITRATE (lame, "bitrate", lame->bitrate, lame->free_format);
   CHECK_ERROR (lame_set_brate (lame->lgf, lame->bitrate));
   CHECK_ERROR (lame_set_compression_ratio (lame->lgf, lame->compression_ratio));
   CHECK_ERROR (lame_set_quality (lame->lgf, lame->quality));
@@ -1024,8 +1182,12 @@ gst_lame_setup (GstLame * lame)
   CHECK_ERROR (lame_set_VBR_q (lame->lgf, lame->vbr_quality));
   CHECK_ERROR (lame_set_VBR_mean_bitrate_kbps (lame->lgf,
           lame->vbr_mean_bitrate));
+  CHECK_AND_FIXUP_BITRATE (lame, "vbr-min-bitrate", lame->vbr_min_bitrate,
+      lame->free_format);
   CHECK_ERROR (lame_set_VBR_min_bitrate_kbps (lame->lgf,
           lame->vbr_min_bitrate));
+  CHECK_AND_FIXUP_BITRATE (lame, "vbr-max-bitrate", lame->vbr_max_bitrate,
+      lame->free_format);
   CHECK_ERROR (lame_set_VBR_max_bitrate_kbps (lame->lgf,
           lame->vbr_max_bitrate));
   CHECK_ERROR (lame_set_VBR_hard_min (lame->lgf, lame->vbr_hard_min));
@@ -1074,7 +1236,9 @@ gst_lame_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      lame->last_flow = GST_FLOW_OK;
       lame->last_ts = GST_CLOCK_TIME_NONE;
+      lame->eos_ts = GST_CLOCK_TIME_NONE;
       break;
     default:
       break;
@@ -1096,15 +1260,22 @@ gst_lame_change_state (GstElement * element, GstStateChange transition)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  GST_DEBUG_CATEGORY_INIT (debug, "lame", 0, "lame mp3 encoder");
+
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   if (!gst_element_register (plugin, "lame", GST_RANK_NONE, GST_TYPE_LAME))
     return FALSE;
 
-  GST_DEBUG_CATEGORY_INIT (debug, "lame", 0, "lame mp3 encoder");
   return TRUE;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "lame",
-    "Encode MP3's with LAME",
+    "Encode MP3s with LAME",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
